/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2010 Richard Hughes <richard@hughsie.com>
 *
 * SPDX-License-Identifier: LGPL-2.1+
 */

#pragma once

#define __GUSB_INSIDE__

#include <gusb/gusb-bos-descriptor.h>
#include <gusb/gusb-context.h>
#include <gusb/gusb-device-list.h>
#include <gusb/gusb-device.h>
#include <gusb/gusb-endpoint.h>
#include <gusb/gusb-interface.h>
#include <gusb/gusb-source.h>
#include <gusb/gusb-util.h>
#include <gusb/gusb-version.h>

#undef __GUSB_INSIDE__
